#ifndef AMGCL_C_DL_H
#define AMGCL_C_DL_H

#ifndef STDINT_INCLUDED
#define STDINT_INCLUDED

#include <stdint.h>

#endif

#ifdef __cplusplus
extern "C" {
#endif
  
#ifndef AMGCLC_INFO
#define AMGCLC_INFO  

#define AMGCL_C_SOLVER_STRUCT struct{void *handle; int blocksize; int error_state;}

  
/** 
    \brief Info structure
*/
  typedef struct {
    int iters;
    double residual;
    int error_state;
  }  amgclcInfo;

  int amgclcBlocksizeInstantiated(int blocksize);

#endif
  
/** 
    \brief algebraic multigrid preconditioned Krylov subspace iterative solver.
    
*/
  typedef AMGCL_C_SOLVER_STRUCT amgclcDLAMGSolver;
  amgclcDLAMGSolver amgclcDLAMGSolverCreate(int64_t n, int64_t *ia, int64_t *ja, double *a, int blocksize, char *params);
  amgclcInfo amgclcDLAMGSolverApply(amgclcDLAMGSolver solver, double *sol, double *rhs);
  void amgclcDLAMGSolverDestroy(amgclcDLAMGSolver solver);
  
/** 
    \brief Single level relaxation preconditioned Krylov subspace iterative solver.
*/
  typedef  AMGCL_C_SOLVER_STRUCT amgclcDLRLXSolver;
  amgclcDLRLXSolver amgclcDLRLXSolverCreate(int64_t n, int64_t *ia, int64_t *ja, double *a, int blocksize, char *params);
  amgclcInfo  amgclcDLRLXSolverApply(amgclcDLRLXSolver solver, double *sol, double *rhs);
  void amgclcDLRLXSolverDestroy(amgclcDLRLXSolver solver);
  
/** 
    \brief One algebraic multigrid preconditioning step.
*/
  typedef  AMGCL_C_SOLVER_STRUCT amgclcDLAMGPrecon;
  amgclcDLAMGPrecon amgclcDLAMGPreconCreate(int64_t n, int64_t *ia, int64_t *ja, double *a, int blocksize, char *params);
  amgclcInfo amgclcDLAMGPreconApply(amgclcDLAMGPrecon solver, double *sol, double *rhs);
  void amgclcDLAMGPreconDestroy(amgclcDLAMGPrecon solver);
  
/** 
    \brief One single level relaxation  preconditioning step.
*/
  typedef  AMGCL_C_SOLVER_STRUCT amgclcDLRLXPrecon;
  amgclcDLRLXPrecon amgclcDLRLXPreconCreate(int64_t n, int64_t *ia, int64_t *ja, double *a, int blocksize, char *params);
  amgclcInfo  amgclcDLRLXPreconApply(amgclcDLRLXPrecon solver, double *sol, double *rhs);
  void amgclcDLRLXPreconDestroy(amgclcDLRLXPrecon solver);
  
  
  
#ifdef __cplusplus
} // extern "C"
#endif

#endif
#ifndef AMGCL_C_DI_H
#define AMGCL_C_DI_H

#ifndef STDINT_INCLUDED
#define STDINT_INCLUDED

#include <stdint.h>

#endif

#ifdef __cplusplus
extern "C" {
#endif
  
#ifndef AMGCLC_INFO
#define AMGCLC_INFO  

#define AMGCL_C_SOLVER_STRUCT struct{void *handle; int blocksize; int error_state;}

  
/** 
    \brief Info structure
*/
  typedef struct {
    int iters;
    double residual;
    int error_state;
  }  amgclcInfo;

  int amgclcBlocksizeInstantiated(int blocksize);

#endif
  
/** 
    \brief algebraic multigrid preconditioned Krylov subspace iterative solver.
    
*/
  typedef AMGCL_C_SOLVER_STRUCT amgclcDIAMGSolver;
  amgclcDIAMGSolver amgclcDIAMGSolverCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  amgclcInfo amgclcDIAMGSolverApply(amgclcDIAMGSolver solver, double *sol, double *rhs);
  void amgclcDIAMGSolverDestroy(amgclcDIAMGSolver solver);
  
/** 
    \brief Single level relaxation preconditioned Krylov subspace iterative solver.
*/
  typedef  AMGCL_C_SOLVER_STRUCT amgclcDIRLXSolver;
  amgclcDIRLXSolver amgclcDIRLXSolverCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  amgclcInfo  amgclcDIRLXSolverApply(amgclcDIRLXSolver solver, double *sol, double *rhs);
  void amgclcDIRLXSolverDestroy(amgclcDIRLXSolver solver);
  
/** 
    \brief One algebraic multigrid preconditioning step.
*/
  typedef  AMGCL_C_SOLVER_STRUCT amgclcDIAMGPrecon;
  amgclcDIAMGPrecon amgclcDIAMGPreconCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  amgclcInfo amgclcDIAMGPreconApply(amgclcDIAMGPrecon solver, double *sol, double *rhs);
  void amgclcDIAMGPreconDestroy(amgclcDIAMGPrecon solver);
  
/** 
    \brief One single level relaxation  preconditioning step.
*/
  typedef  AMGCL_C_SOLVER_STRUCT amgclcDIRLXPrecon;
  amgclcDIRLXPrecon amgclcDIRLXPreconCreate(int n, int *ia, int *ja, double *a, int blocksize, char *params);
  amgclcInfo  amgclcDIRLXPreconApply(amgclcDIRLXPrecon solver, double *sol, double *rhs);
  void amgclcDIRLXPreconDestroy(amgclcDIRLXPrecon solver);
  
  
  
#ifdef __cplusplus
} // extern "C"
#endif

#endif
